@@ -5,6 +5,7 @@
 use std::ptr;
 
 use foreign_types_shared::{ForeignType, ForeignTypeRef};
+use openssl_sys as ffi;
 
 use crate::{cvt, cvt_p, OpenSSLResult};
 
@@ -22,9 +23,12 @@ unsafe impl Sync for Hmac {}
 unsafe impl Send for Hmac {}
 
 impl Hmac {
-    // On BoringSSL, the length is a size_t, so the length conversion is a
+    // On BoringSSL and AWS-LC, the length is a size_t, so the length conversion is a
     // no-op.
-    #[cfg_attr(CRYPTOGRAPHY_IS_BORINGSSL, allow(clippy::useless_conversion))]
+    #[cfg_attr(
+        any(CRYPTOGRAPHY_IS_BORINGSSL, CRYPTOGRAPHY_IS_AWSLC),
+        allow(clippy::useless_conversion)
+    )]
     pub fn new(key: &[u8], md: openssl::hash::MessageDigest) -> OpenSSLResult<Hmac> {
         // SAFETY: All FFI conditions are handled.
         unsafe {
@@ -91,6 +95,8 @@ impl std::ops::Deref for DigestBytes {
 
 #[cfg(test)]
 mod tests {
+    use openssl_sys as ffi;
+
     use super::DigestBytes;
 
     #[test]
