@@ -2,10 +2,11 @@
 // 2.0, and the BSD License. See the LICENSE file in the root of this repository
 // for complete details.
 
+use crate::certificate::SerialNumber;
+use crate::common::Asn1Operation;
 use crate::{common, extensions, name};
 
-pub type ReasonFlags<'a> =
-    Option<common::Asn1ReadableOrWritable<asn1::BitString<'a>, asn1::OwnedBitString>>;
+pub type ReasonFlags<'a, Op> = Option<<Op as Asn1Operation>::OwnedBitString<'a>>;
 
 #[derive(asn1::Asn1Read, asn1::Asn1Write, PartialEq, Eq, Hash)]
 pub struct CertificateRevocationList<'a> {
@@ -35,15 +36,15 @@ pub struct TBSCertList<'a> {
 
 #[derive(asn1::Asn1Read, asn1::Asn1Write, PartialEq, Eq, Hash, Clone)]
 pub struct RevokedCertificate<'a> {
-    pub user_certificate: asn1::BigUint<'a>,
+    pub user_certificate: SerialNumber<'a>,
     pub revocation_date: common::Time,
     pub raw_crl_entry_extensions: Option<extensions::RawExtensions<'a>>,
 }
 
 #[derive(asn1::Asn1Read, asn1::Asn1Write)]
-pub struct IssuingDistributionPoint<'a> {
+pub struct IssuingDistributionPoint<'a, Op: Asn1Operation> {
     #[explicit(0)]
-    pub distribution_point: Option<extensions::DistributionPointName<'a>>,
+    pub distribution_point: Option<extensions::DistributionPointName<'a, Op>>,
 
     #[implicit(1)]
     #[default(false)]
@@ -54,7 +55,7 @@ pub struct IssuingDistributionPoint<'a> {
     pub only_contains_ca_certs: bool,
 
     #[implicit(3)]
-    pub only_some_reasons: ReasonFlags<'a>,
+    pub only_some_reasons: ReasonFlags<'a, Op>,
 
     #[implicit(4)]
     #[default(false)]
