@@ -2,18 +2,18 @@
 // 2.0, and the BSD License. See the LICENSE file in the root of this repository
 // for complete details.
 
-use crate::backend::cipher_registry;
-use crate::buf::{CffiBuf, CffiMutBuf};
-use crate::error::{CryptographyError, CryptographyResult};
-use crate::exceptions;
-use crate::types;
 use pyo3::types::PyAnyMethods;
 use pyo3::IntoPyObject;
 
+use crate::backend::cipher_registry;
+use crate::buf::{CffiBuf, CffiMutBuf};
+use crate::error::{CryptographyError, CryptographyResult};
+use crate::{exceptions, types};
+
 pub(crate) struct CipherContext {
     ctx: openssl::cipher_ctx::CipherCtx,
-    py_mode: pyo3::PyObject,
-    py_algorithm: pyo3::PyObject,
+    py_mode: pyo3::Py<pyo3::PyAny>,
+    py_algorithm: pyo3::Py<pyo3::PyAny>,
     side: openssl::symm::Mode,
 }
 
@@ -495,16 +495,14 @@ impl PyAEADDecryptionContext {
         if tag.len() < min_tag_length {
             return Err(CryptographyError::from(
                 pyo3::exceptions::PyValueError::new_err(format!(
-                    "Authentication tag must be {} bytes or longer.",
-                    min_tag_length
+                    "Authentication tag must be {min_tag_length} bytes or longer.",
                 )),
             ));
         } else if tag.len() > 16 {
             return Err(CryptographyError::from(
-                pyo3::exceptions::PyValueError::new_err(format!(
-                    "Authentication tag cannot be more than {} bytes.",
-                    16
-                )),
+                pyo3::exceptions::PyValueError::new_err(
+                    "Authentication tag cannot be more than 16 bytes.",
+                ),
             ));
         }
 
@@ -610,7 +608,7 @@ fn _advance_aad(ctx: pyo3::Bound<'_, pyo3::PyAny>, n: u64) {
     }
 }
 
-#[pyo3::pymodule]
+#[pyo3::pymodule(gil_used = false)]
 pub(crate) mod ciphers {
     #[pymodule_export]
     use super::{
