@@ -163,7 +163,7 @@ impl Ed25519PublicKey {
     }
 }
 
-#[pyo3::pymodule]
+#[pyo3::pymodule(gil_used = false)]
 pub(crate) mod ed25519 {
     #[pymodule_export]
     use super::{
