@@ -12,6 +12,7 @@ import pytest
 
 from cryptography import utils
 from cryptography.exceptions import InvalidSignature
+from cryptography.hazmat.bindings._rust import openssl as rust_openssl
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import dsa
 from cryptography.hazmat.primitives.asymmetric.utils import (
@@ -408,6 +409,16 @@ class TestDSA:
 
         assert key1 == key2
 
+    def test_private_key_copy(self):
+        key_bytes = load_vectors_from_file(
+            os.path.join("asymmetric", "PKCS8", "unenc-dsa-pkcs8.pem"),
+            lambda pemfile: pemfile.read().encode(),
+        )
+        key1 = serialization.load_pem_private_key(key_bytes, None)
+        key2 = copy.copy(key1)
+
+        assert key1 == key2
+
 
 @pytest.mark.supported(
     only_if=lambda backend: backend.dsa_supported(),
@@ -550,6 +561,27 @@ class TestDSASignature:
         with pytest.raises(ValueError):
             private_key.sign(digest, prehashed_alg)
 
+    @pytest.mark.supported(
+        only_if=lambda _: (
+            rust_openssl.CRYPTOGRAPHY_IS_LIBRESSL
+            or rust_openssl.CRYPTOGRAPHY_IS_BORINGSSL
+            or rust_openssl.CRYPTOGRAPHY_IS_AWSLC
+            or rust_openssl.CRYPTOGRAPHY_OPENSSL_309_OR_GREATER
+        ),
+        skip_message="Requires OpenSSL 3.0.9+, LibreSSL, BoringSSL, or AWS-LC",
+    )
+    def test_nilpotent(self):
+        key = load_vectors_from_file(
+            os.path.join("asymmetric", "DSA", "custom", "nilpotent.pem"),
+            lambda pemfile: serialization.load_pem_private_key(
+                pemfile.read().encode(), password=None
+            ),
+        )
+        assert isinstance(key, dsa.DSAPrivateKey)
+
+        with pytest.raises(ValueError):
+            key.sign(b"anything", hashes.SHA256())
+
 
 class TestDSANumbers:
     def test_dsa_parameter_numbers(self):
