@@ -40,7 +40,6 @@ def should_verify(backend, wycheproof):
 
 
 @wycheproof_tests(
-    "rsa_signature_test.json",
     "rsa_signature_2048_sha224_test.json",
     "rsa_signature_2048_sha256_test.json",
     "rsa_signature_2048_sha384_test.json",
@@ -58,15 +57,19 @@ def should_verify(backend, wycheproof):
     "rsa_signature_3072_sha3_256_test.json",
     "rsa_signature_3072_sha3_384_test.json",
     "rsa_signature_3072_sha3_512_test.json",
+    "rsa_signature_4096_sha256_test.json",
     "rsa_signature_4096_sha384_test.json",
     "rsa_signature_4096_sha512_test.json",
     "rsa_signature_4096_sha512_256_test.json",
+    "rsa_signature_8192_sha256_test.json",
+    "rsa_signature_8192_sha384_test.json",
+    "rsa_signature_8192_sha512_test.json",
 )
 def test_rsa_pkcs1v15_signature(backend, wycheproof):
     key = wycheproof.cache_value_to_group(
         "cached_key",
         lambda: serialization.load_der_public_key(
-            binascii.unhexlify(wycheproof.testgroup["keyDer"]),
+            binascii.unhexlify(wycheproof.testgroup["publicKeyDer"]),
         ),
     )
     assert isinstance(key, rsa.RSAPublicKey)
@@ -94,12 +97,18 @@ def test_rsa_pkcs1v15_signature(backend, wycheproof):
             )
 
 
-@wycheproof_tests("rsa_sig_gen_misc_test.json")
+@wycheproof_tests(
+    "rsa_pkcs1_1024_sig_gen_test.json",
+    "rsa_pkcs1_1536_sig_gen_test.json",
+    "rsa_pkcs1_2048_sig_gen_test.json",
+    "rsa_pkcs1_3072_sig_gen_test.json",
+    "rsa_pkcs1_4096_sig_gen_test.json",
+)
 def test_rsa_pkcs1v15_signature_generation(backend, wycheproof):
     key = wycheproof.cache_value_to_group(
         "cached_key",
-        lambda: serialization.load_pem_private_key(
-            wycheproof.testgroup["privateKeyPem"].encode("ascii"),
+        lambda: serialization.load_der_private_key(
+            binascii.unhexlify(wycheproof.testgroup["privateKeyPkcs8"]),
             password=None,
             unsafe_skip_rsa_key_validation=True,
         ),
@@ -129,7 +138,7 @@ def test_rsa_pkcs1v15_signature_generation(backend, wycheproof):
     "rsa_pss_2048_sha1_mgf1_20_test.json",
     "rsa_pss_2048_sha256_mgf1_0_test.json",
     "rsa_pss_2048_sha256_mgf1_32_test.json",
-    "rsa_pss_2048_sha512_256_mgf1_28_test.json",
+    "rsa_pss_2048_sha512_224_mgf1_28_test.json",
     "rsa_pss_2048_sha512_256_mgf1_32_test.json",
     "rsa_pss_3072_sha256_mgf1_32_test.json",
     "rsa_pss_4096_sha256_mgf1_32_test.json",
@@ -138,18 +147,7 @@ def test_rsa_pkcs1v15_signature_generation(backend, wycheproof):
 )
 def test_rsa_pss_signature(backend, wycheproof):
     digest = _DIGESTS[wycheproof.testgroup["sha"]]
-    if backend._fips_enabled and isinstance(digest, hashes.SHA1):
-        pytest.skip("Invalid params for FIPS. SHA1 is disallowed")
-
-    key = wycheproof.cache_value_to_group(
-        "cached_key",
-        lambda: serialization.load_der_public_key(
-            binascii.unhexlify(wycheproof.testgroup["keyDer"]),
-        ),
-    )
-    assert isinstance(key, rsa.RSAPublicKey)
     mgf_digest = _DIGESTS[wycheproof.testgroup["mgfSha"]]
-
     if digest is None or mgf_digest is None:
         pytest.skip(
             "PSS with digest={} and MGF digest={} not supported".format(
@@ -157,6 +155,23 @@ def test_rsa_pss_signature(backend, wycheproof):
                 wycheproof.testgroup["mgfSha"],
             )
         )
+    if backend._fips_enabled and (
+        isinstance(digest, hashes.SHA1)
+        or isinstance(mgf_digest, hashes.SHA1)
+        # FIPS 186-4 only allows salt length == digest length for PSS
+        or wycheproof.testgroup["sLen"] != mgf_digest.digest_size
+        # inner MGF1 hash must match outer hash
+        or wycheproof.testgroup["sha"] != wycheproof.testgroup["mgfSha"]
+    ):
+        pytest.skip("Invalid params for FIPS")
+
+    key = wycheproof.cache_value_to_group(
+        "cached_key",
+        lambda: serialization.load_der_public_key(
+            binascii.unhexlify(wycheproof.testgroup["publicKeyDer"]),
+        ),
+    )
+    assert isinstance(key, rsa.RSAPublicKey)
 
     if wycheproof.valid or wycheproof.acceptable:
         key.verify(
@@ -202,6 +217,11 @@ def test_rsa_pss_signature(backend, wycheproof):
     "rsa_oaep_misc_test.json",
 )
 def test_rsa_oaep_encryption(backend, wycheproof):
+    if backend._fips_enabled and wycheproof.has_flag("SmallIntegerCiphertext"):
+        pytest.skip(
+            "Small integer ciphertexts are rejected in OpenSSL 3.5 FIPS"
+        )
+
     digest = _DIGESTS[wycheproof.testgroup["sha"]]
     mgf_digest = _DIGESTS[wycheproof.testgroup["mgfSha"]]
     assert digest is not None
@@ -219,8 +239,8 @@ def test_rsa_oaep_encryption(backend, wycheproof):
 
     key = wycheproof.cache_value_to_group(
         "cached_key",
-        lambda: serialization.load_pem_private_key(
-            wycheproof.testgroup["privateKeyPem"].encode("ascii"),
+        lambda: serialization.load_der_private_key(
+            binascii.unhexlify(wycheproof.testgroup["privateKeyPkcs8"]),
             password=None,
             unsafe_skip_rsa_key_validation=True,
         ),
@@ -255,8 +275,8 @@ def test_rsa_oaep_encryption(backend, wycheproof):
 def test_rsa_pkcs1_encryption(backend, wycheproof):
     key = wycheproof.cache_value_to_group(
         "cached_key",
-        lambda: serialization.load_pem_private_key(
-            wycheproof.testgroup["privateKeyPem"].encode("ascii"),
+        lambda: serialization.load_der_private_key(
+            binascii.unhexlify(wycheproof.testgroup["privateKeyPkcs8"]),
             password=None,
             unsafe_skip_rsa_key_validation=True,
         ),
@@ -268,19 +288,18 @@ def test_rsa_pkcs1_encryption(backend, wycheproof):
             binascii.unhexlify(wycheproof.testcase["ct"]), padding.PKCS1v15()
         )
         assert pt == binascii.unhexlify(wycheproof.testcase["msg"])
+    elif backend._lib.Cryptography_HAS_IMPLICIT_RSA_REJECTION:
+        try:
+            assert key.decrypt(
+                binascii.unhexlify(wycheproof.testcase["ct"]),
+                padding.PKCS1v15(),
+            ) != binascii.unhexlify(wycheproof.testcase["ct"])
+        except ValueError:
+            # Some raise ValueError due to length mismatch.
+            pass
     else:
-        if backend._lib.Cryptography_HAS_IMPLICIT_RSA_REJECTION:
-            try:
-                assert key.decrypt(
-                    binascii.unhexlify(wycheproof.testcase["ct"]),
-                    padding.PKCS1v15(),
-                ) != binascii.unhexlify(wycheproof.testcase["ct"])
-            except ValueError:
-                # Some raise ValueError due to length mismatch.
-                pass
-        else:
-            with pytest.raises(ValueError):
-                key.decrypt(
-                    binascii.unhexlify(wycheproof.testcase["ct"]),
-                    padding.PKCS1v15(),
-                )
+        with pytest.raises(ValueError):
+            key.decrypt(
+                binascii.unhexlify(wycheproof.testcase["ct"]),
+                padding.PKCS1v15(),
+            )
